# -*- coding: utf-8 -*-
#
#  ninix-install.py - a command-line installer for ninix
#  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
#  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
#  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#

import getopt
import os
import sys
import logging

logger = logging.getLogger()
logger.setLevel(logging.INFO) # XXX

import gettext
gettext.install('ninix')

import ninix.home
from ninix.install import INTERACTIVE, DOWNLOAD, Installer, InstallError

PROGRAM_NAME = os.path.basename(sys.argv[0])

USAGE = '''\
Usage: %s [options] [file ...]
Options:
  -i, --interactive      prompt before overwrite
  -d, --download         download files into the archive directory
  -r, --rebuild          rebuild ninix home directory (transition from the release of 4.1.3 or older)
  -h, --help             show this message
'''

def usage():
    raise SystemExit, USAGE % PROGRAM_NAME


def main():
    try:
        options, files = getopt.getopt(
            sys.argv[1:], 'idrh',
            ['interactive', 'download', 'rebuild', 'help'])
    except getopt.error, e:
        logging.error('%s' % str(e))
        usage()
    mode = 0
    rebuild = 0
    host = ''
    for opt, val in options:
        if opt in ['-h', '--help']:
            usage()
        elif opt in ['-i', '--interactive']:
            mode = mode | INTERACTIVE
        elif opt in ['-d', '--download']:
            mode = mode | DOWNLOAD
        elif opt in ['-r', '--rebuild']:
            rebuild = 1
    if rebuild:
        mode |= INTERACTIVE
        n = 0
        homedir = ninix.home.get_ninix_home()
        ghost_top = os.path.join(homedir, 'ghost')
        balloon_top = os.path.join(homedir, 'balloon')
        try:
            buf = os.listdir(ghost_top)
        except OSError:
            raise SystemExit
        for subdir in buf:
            ghost_dir = os.path.join(ghost_top, subdir)
            if os.path.isdir(ghost_dir):
                inst = ninix.home.read_install_txt(ghost_dir)
                if inst is None:
                    continue
                balloon_dir = inst.get('balloon.directory')
                if balloon_dir:
                    balloon_dir = ninix.home.get_normalized_path(balloon_dir)
                    path = os.path.join(ghost_dir, 'ghost/master', balloon_dir)
                    if os.path.exists(path):
                        balloon_dst = os.path.join(balloon_top, balloon_dir)
                        if os.path.exists(balloon_dst):
                            # uninstall older versions of the balloon
                            if confirm_removal(balloon_dst):
                                remove_files_and_dirs(balloon_dst, [])
                            else:
                                continue
                        os.rename(path, balloon_dst)
                        # create SETTINGS
                        settings = os.path.join(ghost_dir, 'SETTINGS')
                        if not os.path.exists(settings):
                            try:
                                with open(settings, 'w') as f:
                                    f.write('balloon_directory, %s\n' % balloon_dir)
                            except IOError, (code, message):
                                logging.error('cannot write %s' % settings)
                        n += 1
                ##os.remove(os.path.join(path, 'install.txt'))
        logging.info('%d ghosts updated' % n)
        raise SystemExit
    installer = Installer(mode, gui=0)
    for filename in files:
        try:
            installer.install(filename, ninix.home.get_ninix_home())
        except InstallError, error:
            logging.error('%s: %s (skipped)' % (filename, error))


if __name__ == '__main__':
    main()
